/*
 * Copyright LWJGL. All rights reserved.
 * License terms: https://www.lwjgl.org/license
 * MACHINE GENERATED FILE, DO NOT EDIT
 */
package org.lwjgl.system;

/**
 * This class contains native methods that can be used to call dynamically loaded functions. It is used internally by the LWJGL bindings, but can also
 * be used to call other dynamically loaded functions. Not all possible signatures are available, only those needed by the LWJGL bindings. To call a
 * function that does not have a matching JNI method, {@link org.lwjgl.system.dyncall.DynCall DynCall} can used.
 * 
 * <p>All JNI methods in this class take an extra parameter, called {@code __functionAddress}. This must be a valid pointer to a native function with a
 * matching signature. Due to overloading, method names are partially mangled:</p>
 * 
 * <ul>
 * <li>{@code call} or {@code invoke}
 * 
 * <p>Methods with the {@code invoke} prefix will invoke the native function with the default calling convention. Methods with the {@code call}
 * prefix will invoke the native function with the {@code __stdcall} calling convention on Windows and the default calling convention on other
 * systems.</p></li>
 * <li>a {@code J} or a {@code P} for each {@code long} parameter
 * 
 * <p>{@code J} parameters represent 64-bit integer values. {@code P} parameters represent pointer addresses. A pointer address is a 32-bit value on
 * 32-bit architectures and a 64-bit value on 64-bit architectures.</p></li>
 * <li>the return value <a href="http://docs.oracle.com/javase/8/docs/technotes/guides/jni/spec/types.html#type_signatures">JNI type signature</a></li>
 * </ul>
 */
public final class JNI {

	static {
		Library.initialize();
	}

	private JNI() {}

	// Pointer API

	public static native byte invokePB(long __functionAddress, byte param0, long param1);
	public static native double invokeD(long __functionAddress);
	public static native double invokeD(long __functionAddress, int param0);
	public static native float invokeF(long __functionAddress, int param0);
	public static native int invokeI(long __functionAddress);
	public static native int invokeI(long __functionAddress, byte param0);
	public static native int invokeI(long __functionAddress, byte param0, short param1, int param2, boolean param3);
	public static native int invokeI(long __functionAddress, byte param0, short param1, short param2, int param3, boolean param4);
	public static native int invokeI(long __functionAddress, byte param0, short param1, short param2, short param3, short param4, byte param5);
	public static native int invokeI(long __functionAddress, byte param0, short param1, short param2, short param3, short param4, int param5, boolean param6);
	public static native int invokeI(long __functionAddress, int param0);
	public static native int invokePPI(long __functionAddress, int param0, long param1, int param2, long param3, int param4, boolean param5);
	public static native int invokePI(long __functionAddress, long param0);
	public static native int invokePI(long __functionAddress, long param0, int param1);
	public static native int invokePPI(long __functionAddress, long param0, long param1);
	public static native int invokePPI(long __functionAddress, long param0, long param1, int param2);
	public static native int invokePPPI(long __functionAddress, long param0, long param1, long param2);
	public static native int invokePPPPI(long __functionAddress, long param0, long param1, long param2, long param3);
	public static native int invokePPPPPI(long __functionAddress, long param0, long param1, long param2, long param3, long param4);
	public static native int invokePPPPPPI(long __functionAddress, long param0, long param1, long param2, long param3, long param4, long param5);
	public static native int invokePI(long __functionAddress, long param0, short param1);
	public static native int invokeI(long __functionAddress, short param0);
	public static native int invokePI(long __functionAddress, short param0, byte param1, long param2);
	public static native int invokePI(long __functionAddress, short param0, long param1, byte param2);
	public static native int invokeI(long __functionAddress, boolean param0);
	public static native long invokeJ(long __functionAddress);
	public static native long invokeJJ(long __functionAddress, long param0);
	public static native long invokeJJJ(long __functionAddress, long param0, long param1);
	public static native long invokeJJJJ(long __functionAddress, long param0, long param1, long param2);
	public static native long invokeJJJJJ(long __functionAddress, long param0, long param1, long param2, long param3);
	public static native long invokeP(long __functionAddress);
	public static native long invokeP(long __functionAddress, int param0);
	public static native long invokeP(long __functionAddress, int param0, int param1);
	public static native long invokePPPP(long __functionAddress, int param0, int param1, long param2, long param3, long param4);
	public static native long invokePP(long __functionAddress, int param0, long param1);
	public static native long invokeP(long __functionAddress, int param0, short param1);
	public static native long invokePP(long __functionAddress, long param0);
	public static native long invokePP(long __functionAddress, long param0, int param1);
	public static native long invokePP(long __functionAddress, long param0, int param1, int param2);
	public static native long invokePP(long __functionAddress, long param0, int param1, int param2, int param3);
	public static native long invokePPPP(long __functionAddress, long param0, int param1, long param2, long param3);
	public static native long invokePPP(long __functionAddress, long param0, long param1);
	public static native long invokePPP(long __functionAddress, long param0, long param1, int param2);
	public static native long invokePPPPPP(long __functionAddress, long param0, long param1, int param2, int param3, int param4, int param5, int param6, int param7, int param8, long param9, long param10, long param11);
	public static native long invokePPPP(long __functionAddress, long param0, long param1, long param2);
	public static native long invokePPPP(long __functionAddress, long param0, long param1, long param2, int param3);
	public static native long invokePPPPP(long __functionAddress, long param0, long param1, long param2, long param3);
	public static native long invokePPP(long __functionAddress, long param0, long param1, boolean param2, boolean param3);
	public static native long invokePPP(long __functionAddress, long param0, boolean param1, boolean param2, long param3);
	public static native long invokePP(long __functionAddress, short param0, long param1);
	public static native long invokeP(long __functionAddress, short param0, short param1, short param2, byte param3, int param4, int param5);
	public static native short invokeS(long __functionAddress);
	public static native short invokePS(long __functionAddress, byte param0, long param1, boolean param2);
	public static native short invokeS(long __functionAddress, int param0);
	public static native short invokeS(long __functionAddress, int param0, int param1, int param2);
	public static native short invokePS(long __functionAddress, int param0, long param1, short param2);
	public static native short invokeS(long __functionAddress, int param0, short param1);
	public static native short invokeS(long __functionAddress, int param0, boolean param1, short param2, int param3, int param4);
	public static native short invokePS(long __functionAddress, long param0);
	public static native short invokePPS(long __functionAddress, long param0, int param1, byte param2, long param3);
	public static native short invokePS(long __functionAddress, long param0, int param1, short param2);
	public static native short invokePPPS(long __functionAddress, long param0, long param1, long param2, short param3, float param4);
	public static native short invokePPS(long __functionAddress, long param0, long param1, short param2);
	public static native short invokePS(long __functionAddress, long param0, short param1);
	public static native short invokePS(long __functionAddress, long param0, short param1, short param2, int param3);
	public static native short invokePS(long __functionAddress, short param0, long param1, short param2);
	public static native short invokeS(long __functionAddress, short param0, short param1, int param2, int param3);
	public static native short invokeS(long __functionAddress, short param0, short param1, short param2, short param3);
	public static native short invokePS(long __functionAddress, short param0, short param1, short param2, boolean param3, int param4, int param5, long param6);
	public static native short invokeS(long __functionAddress, short param0, short param1, boolean param2);
	public static native short invokePS(long __functionAddress, short param0, short param1, boolean param2, short param3, int param4, int param5, long param6);
	public static native short invokeS(long __functionAddress, short param0, boolean param1);
	public static native short invokePS(long __functionAddress, short param0, boolean param1, short param2, int param3, int param4, long param5);
	public static native void invokeV(long __functionAddress);
	public static native void invokeV(long __functionAddress, byte param0);
	public static native void invokePV(long __functionAddress, byte param0, byte param1, long param2);
	public static native void invokePV(long __functionAddress, byte param0, long param1);
	public static native void invokePPV(long __functionAddress, byte param0, long param1, long param2);
	public static native void invokePPPV(long __functionAddress, byte param0, long param1, long param2, byte param3, long param4);
	public static native void invokeV(long __functionAddress, byte param0, short param1);
	public static native void invokeV(long __functionAddress, byte param0, short param1, byte param2, short param3, short param4, short param5, short param6, byte param7, byte param8, short param9, short param10, short param11, short param12, short param13, short param14);
	public static native void invokeV(long __functionAddress, byte param0, short param1, byte param2, short param3, short param4, short param5, short param6, byte param7, short param8, short param9, short param10, short param11, short param12, short param13);
	public static native void invokeV(long __functionAddress, byte param0, short param1, float param2, byte param3, byte param4, byte param5, byte param6, byte param7, byte param8, byte param9, byte param10, byte param11);
	public static native void invokeV(long __functionAddress, byte param0, short param1, int param2);
	public static native void invokeV(long __functionAddress, byte param0, short param1, int param2, float param3, byte param4);
	public static native void invokeV(long __functionAddress, byte param0, short param1, short param2, byte param3, int param4);
	public static native void invokeV(long __functionAddress, byte param0, short param1, short param2, byte param3, int param4, int param5);
	public static native void invokeV(long __functionAddress, byte param0, short param1, short param2, int param3);
	public static native void invokeV(long __functionAddress, byte param0, short param1, short param2, short param3, short param4);
	public static native void invokeV(long __functionAddress, byte param0, boolean param1);
	public static native void invokeV(long __functionAddress, double param0);
	public static native void invokeV(long __functionAddress, float param0);
	public static native void invokeV(long __functionAddress, int param0);
	public static native void invokeV(long __functionAddress, int param0, float param1);
	public static native void invokeV(long __functionAddress, int param0, float param1, float param2, float param3);
	public static native void invokeV(long __functionAddress, int param0, int param1);
	public static native void invokeV(long __functionAddress, int param0, int param1, double param2);
	public static native void invokeV(long __functionAddress, int param0, int param1, double param2, double param3, double param4);
	public static native void invokeV(long __functionAddress, int param0, int param1, float param2);
	public static native void invokeV(long __functionAddress, int param0, int param1, float param2, float param3, float param4);
	public static native void invokeV(long __functionAddress, int param0, int param1, int param2);
	public static native void invokeV(long __functionAddress, int param0, int param1, int param2, int param3, int param4);
	public static native void invokePPV(long __functionAddress, int param0, int param1, int param2, long param3, long param4);
	public static native void invokePV(long __functionAddress, int param0, int param1, long param2);
	public static native void invokeJV(long __functionAddress, int param0, int param1, long param2);
	public static native void invokePV(long __functionAddress, int param0, int param1, long param2, int param3, int param4);
	public static native void invokePPPV(long __functionAddress, int param0, int param1, long param2, long param3, long param4);
	public static native void invokeJJJV(long __functionAddress, int param0, int param1, long param2, long param3, long param4);
	public static native void invokePV(long __functionAddress, int param0, long param1);
	public static native void invokePPPPPV(long __functionAddress, int param0, long param1, int param2, long param3, long param4, long param5, int param6, long param7, int param8, boolean param9);
	public static native void invokePPPV(long __functionAddress, int param0, long param1, long param2, long param3);
	public static native void invokeV(long __functionAddress, int param0, short param1);
	public static native void invokePV(long __functionAddress, long param0);
	public static native void invokePV(long __functionAddress, long param0, byte param1);
	public static native void invokePV(long __functionAddress, long param0, double param1, double param2);
	public static native void invokePV(long __functionAddress, long param0, float param1);
	public static native void invokePV(long __functionAddress, long param0, int param1);
	public static native void invokeJV(long __functionAddress, long param0, int param1);
	public static native void invokePV(long __functionAddress, long param0, int param1, byte param2, int param3, boolean param4, boolean param5);
	public static native void invokePV(long __functionAddress, long param0, int param1, int param2);
	public static native void invokePV(long __functionAddress, long param0, int param1, int param2, int param3, int param4);
	public static native void invokePPV(long __functionAddress, long param0, int param1, int param2, long param3);
	public static native void invokePPV(long __functionAddress, long param0, int param1, long param2);
	public static native void invokePPPV(long __functionAddress, long param0, int param1, long param2, long param3);
	public static native void invokePPPV(long __functionAddress, long param0, int param1, long param2, long param3, int param4);
	public static native void invokePPV(long __functionAddress, long param0, long param1);
	public static native void invokePPV(long __functionAddress, long param0, long param1, int param2);
	public static native void invokePPV(long __functionAddress, long param0, long param1, int param2, int param3, int param4, int param5, int param6);
	public static native void invokePPPV(long __functionAddress, long param0, long param1, long param2);
	public static native void invokePPPV(long __functionAddress, long param0, long param1, long param2, int param3);
	public static native void invokePPPV(long __functionAddress, long param0, long param1, long param2, int param3, boolean param4, boolean param5);
	public static native void invokePPPPV(long __functionAddress, long param0, long param1, long param2, long param3);
	public static native void invokePPPPV(long __functionAddress, long param0, long param1, long param2, long param3, int param4);
	public static native void invokePPPPPV(long __functionAddress, long param0, long param1, long param2, long param3, long param4);
	public static native void invokePPPV(long __functionAddress, long param0, long param1, long param2, boolean param3, boolean param4);
	public static native void invokePPV(long __functionAddress, long param0, long param1, boolean param2);
	public static native void invokePV(long __functionAddress, long param0, short param1, short param2, short param3, boolean param4, boolean param5, short param6, int param7);
	public static native void invokePPPV(long __functionAddress, long param0, boolean param1, int param2, long param3, long param4, int param5);
	public static native void invokeV(long __functionAddress, short param0);
	public static native void invokePV(long __functionAddress, short param0, byte param1, short param2, short param3, short param4, short param5, short param6, short param7, long param8);
	public static native void invokeV(long __functionAddress, short param0, int param1, int param2);
	public static native void invokePV(long __functionAddress, short param0, int param1, long param2);
	public static native void invokePV(long __functionAddress, short param0, long param1, short param2);
	public static native void invokePV(long __functionAddress, short param0, short param1, byte param2, byte param3, short param4, short param5, short param6, short param7, long param8, short param9);
	public static native void invokePV(long __functionAddress, short param0, short param1, byte param2, long param3);
	public static native void invokePPV(long __functionAddress, short param0, short param1, byte param2, long param3, long param4);
	public static native void invokePV(long __functionAddress, short param0, short param1, byte param2, short param3, short param4, short param5, short param6, long param7, short param8);
	public static native void invokePV(long __functionAddress, short param0, short param1, short param2, short param3, long param4, short param5);
	public static native void invokeV(long __functionAddress, short param0, boolean param1);
	public static native boolean invokeZ(long __functionAddress, int param0);
	public static native boolean invokePZ(long __functionAddress, int param0, long param1);
	public static native boolean invokePZ(long __functionAddress, int param0, long param1, int param2);
	public static native boolean invokeZ(long __functionAddress, int param0, short param1);
	public static native boolean invokePPZ(long __functionAddress, int param0, short param1, short param2, long param3, long param4);
	public static native boolean invokePZ(long __functionAddress, long param0);
	public static native boolean invokePZ(long __functionAddress, long param0, int param1, int param2, int param3);
	public static native boolean invokePPZ(long __functionAddress, long param0, long param1);
	public static native boolean invokePPPZ(long __functionAddress, long param0, long param1, int param2, long param3, int param4);
	public static native boolean invokePPPPZ(long __functionAddress, long param0, long param1, long param2, byte param3, long param4);
	public static native boolean invokePPPZ(long __functionAddress, long param0, long param1, long param2, int param3);
	public static native boolean invokePPPPZ(long __functionAddress, long param0, long param1, long param2, long param3);
	public static native float callF(long __functionAddress, int param0, int param1, int param2);
	public static native int callI(long __functionAddress);
	public static native int callI(long __functionAddress, int param0);
	public static native int callI(long __functionAddress, int param0, int param1);
	public static native int callI(long __functionAddress, int param0, int param1, int param2);
	public static native int callPI(long __functionAddress, int param0, int param1, int param2, int param3, int param4, long param5);
	public static native int callPI(long __functionAddress, int param0, int param1, int param2, int param3, long param4);
	public static native int callPI(long __functionAddress, int param0, int param1, long param2);
	public static native int callPI(long __functionAddress, int param0, int param1, long param2, int param3, int param4, int param5, int param6, float param7);
	public static native int callPPI(long __functionAddress, int param0, int param1, long param2, long param3, int param4, int param5, int param6, int param7, float param8);
	public static native int callPPPPPI(long __functionAddress, int param0, int param1, long param2, long param3, long param4, long param5, long param6);
	public static native int callPPPPPPI(long __functionAddress, int param0, int param1, long param2, long param3, long param4, long param5, long param6, long param7);
	public static native int callPI(long __functionAddress, int param0, long param1);
	public static native int callPI(long __functionAddress, int param0, long param1, int param2, int param3, float param4, int param5);
	public static native int callPPI(long __functionAddress, int param0, long param1, long param2);
	public static native int callPI(long __functionAddress, long param0);
	public static native int callPI(long __functionAddress, long param0, float param1);
	public static native int callPI(long __functionAddress, long param0, int param1);
	public static native int callPI(long __functionAddress, long param0, int param1, int param2);
	public static native int callPI(long __functionAddress, long param0, int param1, int param2, int param3);
	public static native int callPPI(long __functionAddress, long param0, int param1, int param2, int param3, float param4, float param5, int param6, long param7);
	public static native int callPI(long __functionAddress, long param0, int param1, int param2, int param3, int param4);
	public static native int callPI(long __functionAddress, long param0, int param1, int param2, int param3, int param4, int param5, int param6);
	public static native int callPPI(long __functionAddress, long param0, int param1, int param2, int param3, int param4, int param5, int param6, long param7);
	public static native int callPPI(long __functionAddress, long param0, int param1, int param2, int param3, int param4, int param5, int param6, long param7, int param8, int param9, int param10, int param11, int param12, int param13, int param14, int param15, int param16);
	public static native int callPPI(long __functionAddress, long param0, int param1, int param2, int param3, int param4, int param5, long param6);
	public static native int callPPI(long __functionAddress, long param0, int param1, int param2, int param3, long param4);
	public static native int callPPPI(long __functionAddress, long param0, int param1, int param2, int param3, long param4, long param5);
	public static native int callPPI(long __functionAddress, long param0, int param1, int param2, long param3);
	public static native int callPPPPI(long __functionAddress, long param0, int param1, int param2, long param3, long param4, long param5);
	public static native int callPPI(long __functionAddress, long param0, int param1, long param2);
	public static native int callPJI(long __functionAddress, long param0, int param1, long param2);
	public static native int callPPI(long __functionAddress, long param0, int param1, long param2, int param3);
	public static native int callPPPPI(long __functionAddress, long param0, int param1, long param2, int param3, int param4, long param5, long param6);
	public static native int callPPJI(long __functionAddress, long param0, int param1, long param2, int param3, long param4);
	public static native int callPPPPPI(long __functionAddress, long param0, int param1, long param2, int param3, long param4, int param5, long param6, long param7);
	public static native int callPPPPI(long __functionAddress, long param0, int param1, long param2, int param3, long param4, long param5);
	public static native int callPJPI(long __functionAddress, long param0, int param1, long param2, long param3);
	public static native int callPPJI(long __functionAddress, long param0, int param1, long param2, long param3);
	public static native int callPPPI(long __functionAddress, long param0, int param1, long param2, long param3);
	public static native int callPPJPPI(long __functionAddress, long param0, int param1, long param2, long param3, int param4, long param5, long param6);
	public static native int callPPPPPPPI(long __functionAddress, long param0, int param1, long param2, long param3, int param4, long param5, long param6, long param7, long param8);
	public static native int callPPPPI(long __functionAddress, long param0, int param1, long param2, long param3, long param4);
	public static native int callPJPPPPI(long __functionAddress, long param0, int param1, long param2, long param3, long param4, int param5, long param6, long param7);
	public static native int callPPPPPPI(long __functionAddress, long param0, int param1, long param2, long param3, long param4, int param5, long param6, long param7);
	public static native int callPPPJPPI(long __functionAddress, long param0, int param1, long param2, long param3, long param4, int param5, long param6, long param7);
	public static native int callPPPPPI(long __functionAddress, long param0, int param1, long param2, long param3, long param4, long param5);
	public static native int callPJI(long __functionAddress, long param0, long param1);
	public static native int callPPI(long __functionAddress, long param0, long param1);
	public static native int callPPI(long __functionAddress, long param0, long param1, float param2);
	public static native int callPJI(long __functionAddress, long param0, long param1, int param2);
	public static native int callPPI(long __functionAddress, long param0, long param1, int param2);
	public static native int callPPI(long __functionAddress, long param0, long param1, int param2, int param3);
	public static native int callPPI(long __functionAddress, long param0, long param1, int param2, int param3, int param4);
	public static native int callPPI(long __functionAddress, long param0, long param1, int param2, int param3, int param4, int param5);
	public static native int callPPPI(long __functionAddress, long param0, long param1, int param2, int param3, int param4, int param5, long param6);
	public static native int callPPPI(long __functionAddress, long param0, long param1, int param2, int param3, int param4, long param5);
	public static native int callPPPPI(long __functionAddress, long param0, long param1, int param2, int param3, long param4, long param5);
	public static native int callPJPPI(long __functionAddress, long param0, long param1, int param2, int param3, long param4, long param5);
	public static native int callPJPPJI(long __functionAddress, long param0, long param1, int param2, int param3, long param4, long param5, long param6, int param7);
	public static native int callPPJI(long __functionAddress, long param0, long param1, int param2, long param3);
	public static native int callPPPI(long __functionAddress, long param0, long param1, int param2, long param3);
	public static native int callPJPI(long __functionAddress, long param0, long param1, int param2, long param3);
	public static native int callPPJPPI(long __functionAddress, long param0, long param1, int param2, long param3, int param4, long param5, long param6);
	public static native int callPJPPI(long __functionAddress, long param0, long param1, int param2, long param3, long param4);
	public static native int callPPPPI(long __functionAddress, long param0, long param1, int param2, long param3, long param4);
	public static native int callPJPPPI(long __functionAddress, long param0, long param1, int param2, long param3, long param4, long param5);
	public static native int callPPPPPI(long __functionAddress, long param0, long param1, int param2, long param3, long param4, long param5);
	public static native int callPPPPPPPI(long __functionAddress, long param0, long param1, int param2, long param3, long param4, long param5, int param6, long param7, long param8);
	public static native int callPPPPPPPI(long __functionAddress, long param0, long param1, int param2, long param3, long param4, long param5, long param6, long param7);
	public static native int callPPPPPPPPPI(long __functionAddress, long param0, long param1, int param2, long param3, long param4, long param5, long param6, long param7, int param8, long param9, long param10);
	public static native int callPPPPPPPPPPPPI(long __functionAddress, long param0, long param1, int param2, long param3, long param4, long param5, long param6, long param7, long param8, long param9, long param10, int param11, long param12, long param13);
	public static native int callPJPI(long __functionAddress, long param0, long param1, long param2);
	public static native int callPPJI(long __functionAddress, long param0, long param1, long param2);
	public static native int callPPPI(long __functionAddress, long param0, long param1, long param2);
	public static native int callPPPI(long __functionAddress, long param0, long param1, long param2, int param3);
	public static native int callPPPPI(long __functionAddress, long param0, long param1, long param2, int param3, long param4);
	public static native int callPPPPPI(long __functionAddress, long param0, long param1, long param2, int param3, long param4, long param5);
	public static native int callPJJJI(long __functionAddress, long param0, long param1, long param2, long param3);
	public static native int callPPPPI(long __functionAddress, long param0, long param1, long param2, long param3);
	public static native int callPJPPI(long __functionAddress, long param0, long param1, long param2, long param3);
	public static native int callPJJJPI(long __functionAddress, long param0, long param1, long param2, long param3, int param4, long param5);
	public static native int callPPPPPPPPI(long __functionAddress, long param0, long param1, long param2, long param3, int param4, long param5, long param6, int param7, long param8, long param9);
	public static native int callPPPPPPPI(long __functionAddress, long param0, long param1, long param2, long param3, int param4, long param5, long param6, long param7);
	public static native int callPJPPPI(long __functionAddress, long param0, long param1, long param2, long param3, long param4);
	public static native int callPPPPPI(long __functionAddress, long param0, long param1, long param2, long param3, long param4);
	public static native int callPPPPPPPI(long __functionAddress, long param0, long param1, long param2, long param3, long param4, int param5, long param6, long param7);
	public static native int callPJJJJPI(long __functionAddress, long param0, long param1, long param2, long param3, long param4, long param5);
	public static native int callPPPPPPI(long __functionAddress, long param0, long param1, long param2, long param3, long param4, long param5);
	public static native int callPPPPPPPPI(long __functionAddress, long param0, long param1, long param2, long param3, long param4, long param5, int param6, long param7, long param8);
	public static native int callPPPPPPPPPPPPI(long __functionAddress, long param0, long param1, long param2, long param3, long param4, long param5, long param6, long param7, long param8, long param9, int param10, long param11, long param12);
	public static native long callJ(long __functionAddress);
	public static native long callJ(long __functionAddress, int param0);
	public static native long callJ(long __functionAddress, int param0, int param1);
	public static native long callJ(long __functionAddress, int param0, int param1, boolean param2, int param3, int param4);
	public static native long callP(long __functionAddress);
	public static native long callP(long __functionAddress, int param0);
	public static native long callP(long __functionAddress, int param0, float param1, float param2, float param3);
	public static native long callP(long __functionAddress, int param0, int param1);
	public static native long callPPP(long __functionAddress, int param0, int param1, int param2, long param3, long param4);
	public static native long callPP(long __functionAddress, int param0, long param1);
	public static native long callPP(long __functionAddress, int param0, long param1, int param2);
	public static native long callPPP(long __functionAddress, int param0, long param1, long param2);
	public static native long callPPP(long __functionAddress, int param0, long param1, long param2, int param3);
	public static native long callPP(long __functionAddress, long param0);
	public static native long callPP(long __functionAddress, long param0, int param1);
	public static native long callPP(long __functionAddress, long param0, int param1, int param2);
	public static native long callPPP(long __functionAddress, long param0, int param1, int param2, int param3, long param4);
	public static native long callPPP(long __functionAddress, long param0, int param1, long param2);
	public static native long callPPPP(long __functionAddress, long param0, int param1, long param2, long param3);
	public static native long callPPPPPPP(long __functionAddress, long param0, int param1, long param2, long param3, int param4, long param5, long param6, long param7);
	public static native long callPPPPP(long __functionAddress, long param0, int param1, long param2, long param3, long param4);
	public static native long callPPPPPP(long __functionAddress, long param0, int param1, long param2, long param3, long param4, long param5);
	public static native long callPPPPPPP(long __functionAddress, long param0, int param1, long param2, long param3, long param4, long param5, long param6);
	public static native long callPPP(long __functionAddress, long param0, long param1);
	public static native long callPPP(long __functionAddress, long param0, long param1, int param2);
	public static native long callPPP(long __functionAddress, long param0, long param1, int param2, int param3, int param4);
	public static native long callPJPP(long __functionAddress, long param0, long param1, int param2, int param3, int param4, long param5);
	public static native long callPPPP(long __functionAddress, long param0, long param1, int param2, int param3, long param4);
	public static native long callPJPPP(long __functionAddress, long param0, long param1, int param2, int param3, long param4, long param5);
	public static native long callPPPP(long __functionAddress, long param0, long param1, int param2, long param3);
	public static native long callPJPP(long __functionAddress, long param0, long param1, int param2, long param3);
	public static native long callPPPP(long __functionAddress, long param0, long param1, int param2, long param3, int param4);
	public static native long callPJPPP(long __functionAddress, long param0, long param1, int param2, long param3, long param4);
	public static native long callPPPPP(long __functionAddress, long param0, long param1, int param2, long param3, long param4);
	public static native long callPPJPPPPPP(long __functionAddress, long param0, long param1, int param2, long param3, long param4, long param5, int param6, long param7, long param8, long param9);
	public static native long callPPJPPPPPPPP(long __functionAddress, long param0, long param1, int param2, long param3, long param4, long param5, long param6, long param7, int param8, long param9, long param10, long param11);
	public static native long callPPPP(long __functionAddress, long param0, long param1, long param2);
	public static native long callPJPP(long __functionAddress, long param0, long param1, long param2, int param3);
	public static native long callPPPP(long __functionAddress, long param0, long param1, long param2, int param3);
	public static native long callPPPPP(long __functionAddress, long param0, long param1, long param2, int param3, long param4);
	public static native long callPJPPP(long __functionAddress, long param0, long param1, long param2, int param3, long param4);
	public static native long callPPPPP(long __functionAddress, long param0, long param1, long param2, long param3);
	public static native long callPPJPP(long __functionAddress, long param0, long param1, long param2, long param3);
	public static native long callPJPPPP(long __functionAddress, long param0, long param1, long param2, long param3, long param4);
	public static native long callPJPPPPP(long __functionAddress, long param0, long param1, long param2, long param3, long param4, long param5);
	public static native long callPPPJPPP(long __functionAddress, long param0, long param1, long param2, long param3, long param4, long param5);
	public static native long callPJPPPPPPP(long __functionAddress, long param0, long param1, long param2, long param3, long param4, long param5, long param6, long param7);
	public static native long callPJPPPPPPPPP(long __functionAddress, long param0, long param1, long param2, long param3, long param4, long param5, long param6, long param7, long param8, long param9);
	public static native short callS(long __functionAddress, int param0);
	public static native void callV(long __functionAddress);
	public static native void callV(long __functionAddress, byte param0);
	public static native void callV(long __functionAddress, byte param0, byte param1, byte param2);
	public static native void callV(long __functionAddress, byte param0, byte param1, byte param2, byte param3);
	public static native void callV(long __functionAddress, double param0);
	public static native void callV(long __functionAddress, double param0, double param1);
	public static native void callV(long __functionAddress, double param0, double param1, double param2);
	public static native void callV(long __functionAddress, double param0, double param1, double param2, double param3);
	public static native void callV(long __functionAddress, double param0, double param1, double param2, double param3, double param4, double param5);
	public static native void callV(long __functionAddress, float param0);
	public static native void callV(long __functionAddress, float param0, float param1);
	public static native void callV(long __functionAddress, float param0, float param1, float param2);
	public static native void callV(long __functionAddress, float param0, float param1, float param2, float param3);
	public static native void callV(long __functionAddress, float param0, float param1, float param2, float param3, float param4, float param5, float param6, float param7);
	public static native void callV(long __functionAddress, float param0, boolean param1);
	public static native void callV(long __functionAddress, int param0);
	public static native void callV(long __functionAddress, int param0, byte param1, byte param2, byte param3, byte param4);
	public static native void callV(long __functionAddress, int param0, double param1);
	public static native void callV(long __functionAddress, int param0, double param1, double param2);
	public static native void callV(long __functionAddress, int param0, double param1, double param2, double param3);
	public static native void callV(long __functionAddress, int param0, double param1, double param2, double param3, double param4);
	public static native void callV(long __functionAddress, int param0, double param1, double param2, double param3, double param4, double param5, double param6);
	public static native void callV(long __functionAddress, int param0, double param1, double param2, int param3, double param4, double param5);
	public static native void callPV(long __functionAddress, int param0, double param1, double param2, int param3, int param4, double param5, double param6, int param7, int param8, long param9);
	public static native void callPV(long __functionAddress, int param0, double param1, double param2, int param3, int param4, long param5);
	public static native void callV(long __functionAddress, int param0, float param1);
	public static native void callV(long __functionAddress, int param0, float param1, float param2);
	public static native void callV(long __functionAddress, int param0, float param1, float param2, float param3);
	public static native void callV(long __functionAddress, int param0, float param1, float param2, float param3, float param4);
	public static native void callV(long __functionAddress, int param0, float param1, float param2, int param3, float param4, float param5);
	public static native void callPV(long __functionAddress, int param0, float param1, float param2, int param3, int param4, float param5, float param6, int param7, int param8, long param9);
	public static native void callPV(long __functionAddress, int param0, float param1, float param2, int param3, int param4, long param5);
	public static native void callV(long __functionAddress, int param0, int param1);
	public static native void callV(long __functionAddress, int param0, int param1, double param2);
	public static native void callV(long __functionAddress, int param0, int param1, double param2, double param3);
	public static native void callV(long __functionAddress, int param0, int param1, double param2, double param3, double param4);
	public static native void callV(long __functionAddress, int param0, int param1, double param2, double param3, double param4, double param5);
	public static native void callV(long __functionAddress, int param0, int param1, float param2);
	public static native void callV(long __functionAddress, int param0, int param1, float param2, float param3);
	public static native void callV(long __functionAddress, int param0, int param1, float param2, float param3, float param4);
	public static native void callV(long __functionAddress, int param0, int param1, float param2, float param3, float param4, float param5);
	public static native void callV(long __functionAddress, int param0, int param1, float param2, float param3, float param4, float param5, float param6, float param7, float param8, float param9, float param10);
	public static native void callPV(long __functionAddress, int param0, int param1, float param2, float param3, float param4, float param5, long param6);
	public static native void callV(long __functionAddress, int param0, int param1, float param2, int param3);
	public static native void callV(long __functionAddress, int param0, int param1, int param2);
	public static native void callV(long __functionAddress, int param0, int param1, int param2, double param3);
	public static native void callV(long __functionAddress, int param0, int param1, int param2, double param3, double param4, double param5, double param6);
	public static native void callV(long __functionAddress, int param0, int param1, int param2, float param3);
	public static native void callV(long __functionAddress, int param0, int param1, int param2, float param3, float param4, float param5, float param6);
	public static native void callV(long __functionAddress, int param0, int param1, int param2, float param3, int param4);
	public static native void callV(long __functionAddress, int param0, int param1, int param2, int param3);
	public static native void callV(long __functionAddress, int param0, int param1, int param2, int param3, int param4);
	public static native void callV(long __functionAddress, int param0, int param1, int param2, int param3, int param4, int param5);
	public static native void callV(long __functionAddress, int param0, int param1, int param2, int param3, int param4, int param5, int param6);
	public static native void callV(long __functionAddress, int param0, int param1, int param2, int param3, int param4, int param5, int param6, int param7);
	public static native void callV(long __functionAddress, int param0, int param1, int param2, int param3, int param4, int param5, int param6, int param7, int param8);
	public static native void callV(long __functionAddress, int param0, int param1, int param2, int param3, int param4, int param5, int param6, int param7, int param8, int param9);
	public static native void callV(long __functionAddress, int param0, int param1, int param2, int param3, int param4, int param5, int param6, int param7, int param8, int param9, int param10, int param11);
	public static native void callV(long __functionAddress, int param0, int param1, int param2, int param3, int param4, int param5, int param6, int param7, int param8, int param9, int param10, int param11, int param12, int param13, int param14);
	public static native void callV(long __functionAddress, int param0, int param1, int param2, int param3, int param4, int param5, int param6, int param7, int param8, int param9, int param10, int param11, int param12, int param13, int param14, int param15, int param16);
	public static native void callPV(long __functionAddress, int param0, int param1, int param2, int param3, int param4, int param5, int param6, int param7, int param8, int param9, int param10, long param11);
	public static native void callPV(long __functionAddress, int param0, int param1, int param2, int param3, int param4, int param5, int param6, int param7, int param8, int param9, long param10);
	public static native void callPV(long __functionAddress, int param0, int param1, int param2, int param3, int param4, int param5, int param6, int param7, int param8, long param9);
	public static native void callPV(long __functionAddress, int param0, int param1, int param2, int param3, int param4, int param5, int param6, int param7, long param8);
	public static native void callV(long __functionAddress, int param0, int param1, int param2, int param3, int param4, int param5, int param6, int param7, boolean param8);
	public static native void callPV(long __functionAddress, int param0, int param1, int param2, int param3, int param4, int param5, int param6, long param7);
	public static native void callV(long __functionAddress, int param0, int param1, int param2, int param3, int param4, int param5, int param6, boolean param7);
	public static native void callPV(long __functionAddress, int param0, int param1, int param2, int param3, int param4, int param5, long param6);
	public static native void callPPV(long __functionAddress, int param0, int param1, int param2, int param3, int param4, int param5, long param6, long param7);
	public static native void callV(long __functionAddress, int param0, int param1, int param2, int param3, int param4, int param5, boolean param6);
	public static native void callPV(long __functionAddress, int param0, int param1, int param2, int param3, int param4, long param5);
	public static native void callPV(long __functionAddress, int param0, int param1, int param2, int param3, int param4, long param5, int param6);
	public static native void callV(long __functionAddress, int param0, int param1, int param2, int param3, int param4, boolean param5);
	public static native void callPV(long __functionAddress, int param0, int param1, int param2, int param3, int param4, boolean param5, int param6, long param7);
	public static native void callPV(long __functionAddress, int param0, int param1, int param2, int param3, long param4);
	public static native void callPPV(long __functionAddress, int param0, int param1, int param2, int param3, long param4, int param5, int param6, long param7);
	public static native void callPPPV(long __functionAddress, int param0, int param1, int param2, int param3, long param4, int param5, long param6, long param7);
	public static native void callPPV(long __functionAddress, int param0, int param1, int param2, int param3, long param4, long param5);
	public static native void callPPPV(long __functionAddress, int param0, int param1, int param2, int param3, long param4, long param5, long param6);
	public static native void callPV(long __functionAddress, int param0, int param1, int param2, int param3, long param4, boolean param5);
	public static native void callV(long __functionAddress, int param0, int param1, int param2, int param3, boolean param4, int param5);
	public static native void callPV(long __functionAddress, int param0, int param1, int param2, long param3);
	public static native void callPV(long __functionAddress, int param0, int param1, int param2, long param3, int param4);
	public static native void callPPV(long __functionAddress, int param0, int param1, int param2, long param3, int param4, float param5, float param6, int param7, long param8);
	public static native void callPV(long __functionAddress, int param0, int param1, int param2, long param3, int param4, int param5);
	public static native void callPV(long __functionAddress, int param0, int param1, int param2, long param3, int param4, int param5, int param6);
	public static native void callPPV(long __functionAddress, int param0, int param1, int param2, long param3, int param4, int param5, long param6);
	public static native void callPPPPPPPV(long __functionAddress, int param0, int param1, int param2, long param3, int param4, long param5, long param6, long param7, long param8, long param9, long param10);
	public static native void callPPV(long __functionAddress, int param0, int param1, int param2, long param3, long param4);
	public static native void callPPPV(long __functionAddress, int param0, int param1, int param2, long param3, long param4, long param5);
	public static native void callPPPPV(long __functionAddress, int param0, int param1, int param2, long param3, long param4, long param5, long param6);
	public static native void callPV(long __functionAddress, int param0, int param1, int param2, long param3, boolean param4);
	public static native void callV(long __functionAddress, int param0, int param1, int param2, boolean param3);
	public static native void callV(long __functionAddress, int param0, int param1, int param2, boolean param3, int param4);
	public static native void callV(long __functionAddress, int param0, int param1, int param2, boolean param3, int param4, int param5, int param6);
	public static native void callPV(long __functionAddress, int param0, int param1, int param2, boolean param3, int param4, long param5);
	public static native void callPV(long __functionAddress, int param0, int param1, int param2, boolean param3, long param4);
	public static native void callPV(long __functionAddress, int param0, int param1, long param2);
	public static native void callJV(long __functionAddress, int param0, int param1, long param2);
	public static native void callPV(long __functionAddress, int param0, int param1, long param2, int param3);
	public static native void callPV(long __functionAddress, int param0, int param1, long param2, int param3, int param4);
	public static native void callPV(long __functionAddress, int param0, int param1, long param2, int param3, int param4, int param5);
	public static native void callPV(long __functionAddress, int param0, int param1, long param2, int param3, int param4, int param5, int param6);
	public static native void callPV(long __functionAddress, int param0, int param1, long param2, int param3, int param4, int param5, int param6, int param7, float param8);
	public static native void callPPV(long __functionAddress, int param0, int param1, long param2, int param3, int param4, int param5, int param6, int param7, long param8);
	public static native void callPPV(long __functionAddress, int param0, int param1, long param2, int param3, int param4, int param5, int param6, long param7);
	public static native void callPPV(long __functionAddress, int param0, int param1, long param2, int param3, int param4, int param5, long param6);
	public static native void callPPV(long __functionAddress, int param0, int param1, long param2, int param3, int param4, int param5, long param6, int param7, int param8, float param9);
	public static native void callPPV(long __functionAddress, int param0, int param1, long param2, int param3, int param4, long param5);
	public static native void callPPV(long __functionAddress, int param0, int param1, long param2, int param3, long param4);
	public static native void callJJV(long __functionAddress, int param0, int param1, long param2, long param3);
	public static native void callPPV(long __functionAddress, int param0, int param1, long param2, long param3);
	public static native void callJPV(long __functionAddress, int param0, int param1, long param2, long param3);
	public static native void callPPV(long __functionAddress, int param0, int param1, long param2, long param3, int param4);
	public static native void callPPV(long __functionAddress, int param0, int param1, long param2, long param3, int param4, int param5);
	public static native void callPPV(long __functionAddress, int param0, int param1, long param2, long param3, int param4, int param5, int param6);
	public static native void callPPPV(long __functionAddress, int param0, int param1, long param2, long param3, int param4, int param5, long param6);
	public static native void callPPPV(long __functionAddress, int param0, int param1, long param2, long param3, long param4);
	public static native void callJJJV(long __functionAddress, int param0, int param1, long param2, long param3, long param4);
	public static native void callJJJJV(long __functionAddress, int param0, int param1, long param2, long param3, long param4, long param5);
	public static native void callPPPPV(long __functionAddress, int param0, int param1, long param2, long param3, long param4, long param5, int param6);
	public static native void callPPPPPV(long __functionAddress, int param0, int param1, long param2, long param3, long param4, long param5, long param6);
	public static native void callV(long __functionAddress, int param0, int param1, boolean param2);
	public static native void callV(long __functionAddress, int param0, int param1, boolean param2, int param3);
	public static native void callPV(long __functionAddress, int param0, int param1, boolean param2, long param3);
	public static native void callPV(long __functionAddress, int param0, long param1);
	public static native void callJV(long __functionAddress, int param0, long param1);
	public static native void callPV(long __functionAddress, int param0, long param1, int param2);
	public static native void callPV(long __functionAddress, int param0, long param1, int param2, int param3);
	public static native void callPV(long __functionAddress, int param0, long param1, int param2, int param3, int param4);
	public static native void callPPV(long __functionAddress, int param0, long param1, int param2, long param3);
	public static native void callPPV(long __functionAddress, int param0, long param1, int param2, long param3, int param4);
	public static native void callPPPV(long __functionAddress, int param0, long param1, int param2, long param3, int param4, long param5);
	public static native void callPPPV(long __functionAddress, int param0, long param1, int param2, long param3, long param4);
	public static native void callPPV(long __functionAddress, int param0, long param1, long param2);
	public static native void callJJV(long __functionAddress, int param0, long param1, long param2);
	public static native void callPPV(long __functionAddress, int param0, long param1, long param2, int param3);
	public static native void callPPV(long __functionAddress, int param0, long param1, long param2, int param3, int param4);
	public static native void callPPV(long __functionAddress, int param0, long param1, long param2, int param3, int param4, int param5);
	public static native void callPPPV(long __functionAddress, int param0, long param1, long param2, int param3, long param4);
	public static native void callPPPV(long __functionAddress, int param0, long param1, long param2, long param3);
	public static native void callJJJV(long __functionAddress, int param0, long param1, long param2, long param3);
	public static native void callJJJJV(long __functionAddress, int param0, long param1, long param2, long param3, long param4);
	public static native void callPPPPV(long __functionAddress, int param0, long param1, long param2, long param3, long param4, int param5);
	public static native void callPPV(long __functionAddress, int param0, long param1, long param2, boolean param3);
	public static native void callV(long __functionAddress, int param0, short param1);
	public static native void callV(long __functionAddress, int param0, short param1, short param2);
	public static native void callV(long __functionAddress, int param0, short param1, short param2, short param3);
	public static native void callV(long __functionAddress, int param0, short param1, short param2, short param3, short param4);
	public static native void callV(long __functionAddress, int param0, boolean param1);
	public static native void callPV(long __functionAddress, int param0, boolean param1, int param2, int param3, int param4, long param5);
	public static native void callPV(long __functionAddress, int param0, boolean param1, int param2, int param3, long param4);
	public static native void callV(long __functionAddress, int param0, boolean param1, boolean param2, boolean param3, boolean param4);
	public static native void callJV(long __functionAddress, long param0);
	public static native void callPV(long __functionAddress, long param0);
	public static native void callPV(long __functionAddress, long param0, float param1);
	public static native void callPV(long __functionAddress, long param0, float param1, float param2);
	public static native void callPV(long __functionAddress, long param0, float param1, float param2, float param3);
	public static native void callPV(long __functionAddress, long param0, int param1);
	public static native void callJV(long __functionAddress, long param0, int param1);
	public static native void callJV(long __functionAddress, long param0, int param1, float param2, float param3, float param4, float param5, float param6, float param7, float param8, float param9, float param10);
	public static native void callPV(long __functionAddress, long param0, int param1, int param2);
	public static native void callPV(long __functionAddress, long param0, int param1, int param2, int param3);
	public static native void callPV(long __functionAddress, long param0, int param1, int param2, int param3, int param4);
	public static native void callPV(long __functionAddress, long param0, int param1, int param2, int param3, int param4, int param5);
	public static native void callPV(long __functionAddress, long param0, int param1, int param2, int param3, int param4, int param5, int param6, int param7, int param8, int param9, int param10);
	public static native void callPPPV(long __functionAddress, long param0, int param1, int param2, int param3, int param4, int param5, long param6, long param7);
	public static native void callPPPPV(long __functionAddress, long param0, int param1, int param2, int param3, int param4, long param5, int param6, long param7, int param8, long param9);
	public static native void callPPV(long __functionAddress, long param0, int param1, int param2, long param3);
	public static native void callPPPV(long __functionAddress, long param0, int param1, int param2, long param3, long param4);
	public static native void callPJPPPV(long __functionAddress, long param0, int param1, int param2, long param3, long param4, int param5, long param6, long param7);
	public static native void callPPV(long __functionAddress, long param0, int param1, long param2);
	public static native void callPJV(long __functionAddress, long param0, int param1, long param2);
	public static native void callPJV(long __functionAddress, long param0, int param1, long param2, int param3);
	public static native void callPPPPPV(long __functionAddress, long param0, int param1, long param2, int param3, int param4, int param5, long param6, int param7, long param8, int param9, long param10);
	public static native void callPJPPV(long __functionAddress, long param0, int param1, long param2, int param3, int param4, long param5, int param6, long param7);
	public static native void callPPPV(long __functionAddress, long param0, int param1, long param2, int param3, long param4);
	public static native void callPPPV(long __functionAddress, long param0, int param1, long param2, long param3);
	public static native void callPPV(long __functionAddress, long param0, long param1);
	public static native void callPJV(long __functionAddress, long param0, long param1);
	public static native void callPPV(long __functionAddress, long param0, long param1, int param2);
	public static native void callPJV(long __functionAddress, long param0, long param1, int param2);
	public static native void callPJV(long __functionAddress, long param0, long param1, int param2, int param3);
	public static native void callPPPV(long __functionAddress, long param0, long param1, int param2, int param3, int param4, int param5, int param6, int param7, long param8, int param9, int param10, int param11, int param12, int param13, int param14, int param15, int param16, int param17);
	public static native void callPJPV(long __functionAddress, long param0, long param1, int param2, int param3, int param4, long param5);
	public static native void callPJJJJV(long __functionAddress, long param0, long param1, int param2, int param3, long param4, long param5, long param6, int param7);
	public static native void callPJPV(long __functionAddress, long param0, long param1, int param2, long param3);
	public static native void callPPPV(long __functionAddress, long param0, long param1, int param2, long param3);
	public static native void callPJJPV(long __functionAddress, long param0, long param1, int param2, long param3, int param4, int param5, long param6);
	public static native void callPJJPV(long __functionAddress, long param0, long param1, int param2, long param3, int param4, int param5, long param6, int param7);
	public static native void callPJJPV(long __functionAddress, long param0, long param1, int param2, long param3, int param4, long param5);
	public static native void callPJPPV(long __functionAddress, long param0, long param1, int param2, long param3, int param4, long param5);
	public static native void callPJPV(long __functionAddress, long param0, long param1, long param2);
	public static native void callPPPV(long __functionAddress, long param0, long param1, long param2);
	public static native void callPJJV(long __functionAddress, long param0, long param1, long param2);
	public static native void callPJJV(long __functionAddress, long param0, long param1, long param2, int param3);
	public static native void callPJJV(long __functionAddress, long param0, long param1, long param2, int param3, int param4);
	public static native void callPJJPV(long __functionAddress, long param0, long param1, long param2, int param3, int param4, long param5);
	public static native void callPPPPPPV(long __functionAddress, long param0, long param1, long param2, int param3, int param4, long param5, long param6, long param7);
	public static native void callPJJPV(long __functionAddress, long param0, long param1, long param2, int param3, long param4);
	public static native void callPJPPV(long __functionAddress, long param0, long param1, long param2, long param3);
	public static native void callPPPPV(long __functionAddress, long param0, long param1, long param2, long param3);
	public static native void callPJJJV(long __functionAddress, long param0, long param1, long param2, long param3, int param4);
	public static native void callPPPPV(long __functionAddress, long param0, long param1, long param2, long param3, int param4);
	public static native void callPJJJPV(long __functionAddress, long param0, long param1, long param2, long param3, long param4);
	public static native void callPJJJJV(long __functionAddress, long param0, long param1, long param2, long param3, long param4, int param5, int param6);
	public static native void callV(long __functionAddress, short param0);
	public static native void callV(long __functionAddress, short param0, short param1);
	public static native void callV(long __functionAddress, short param0, short param1, short param2);
	public static native void callV(long __functionAddress, short param0, short param1, short param2, short param3);
	public static native void callV(long __functionAddress, boolean param0);
	public static native void callV(long __functionAddress, boolean param0, boolean param1, boolean param2, boolean param3);
	public static native boolean callZ(long __functionAddress, int param0);
	public static native boolean callZ(long __functionAddress, int param0, float param1, float param2);
	public static native boolean callZ(long __functionAddress, int param0, int param1);
	public static native boolean callZ(long __functionAddress, int param0, int param1, float param2, float param3);
	public static native boolean callPPPPZ(long __functionAddress, int param0, int param1, int param2, float param3, long param4, long param5, long param6, long param7);
	public static native boolean callPZ(long __functionAddress, int param0, long param1);
	public static native boolean callPPZ(long __functionAddress, int param0, long param1, long param2);
	public static native boolean callPZ(long __functionAddress, long param0);
	public static native boolean callJZ(long __functionAddress, long param0);

	// Array API

	public static native byte invokePB(long __functionAddress, byte param0, int[] param1);
	public static native int invokePPI(long __functionAddress, int param0, int[] param1, int param2, int[] param3, int param4, boolean param5);
	public static native int invokePPI(long __functionAddress, int param0, short[] param1, int param2, short[] param3, int param4, boolean param5);
	public static native int invokePPPPI(long __functionAddress, long param0, long param1, long param2, long[] param3);
	public static native int invokePI(long __functionAddress, short param0, byte param1, float[] param2);
	public static native int invokePI(long __functionAddress, short param0, byte param1, int[] param2);
	public static native int invokePI(long __functionAddress, short param0, byte param1, short[] param2);
	public static native int invokePI(long __functionAddress, short param0, float[] param1, byte param2);
	public static native int invokePI(long __functionAddress, short param0, int[] param1, byte param2);
	public static native int invokePI(long __functionAddress, short param0, short[] param1, byte param2);
	public static native int invokePI(long __functionAddress, float[] param0, short param1);
	public static native long invokePPP(long __functionAddress, long param0, int[] param1);
	public static native long invokePP(long __functionAddress, double[] param0, int param1);
	public static native long invokePP(long __functionAddress, float[] param0, int param1);
	public static native long invokePP(long __functionAddress, int[] param0, int param1);
	public static native long invokePP(long __functionAddress, long[] param0, int param1);
	public static native long invokePP(long __functionAddress, short[] param0, int param1);
	public static native short invokePS(long __functionAddress, byte param0, short[] param1, boolean param2);
	public static native short invokePS(long __functionAddress, short param0, short[] param1, short param2);
	public static native short invokePPPS(long __functionAddress, short[] param0, long param1, long param2, short param3, float param4);
	public static native void invokePV(long __functionAddress, byte param0, float[] param1);
	public static native void invokePPV(long __functionAddress, byte param0, float[] param1, float[] param2);
	public static native void invokePPPV(long __functionAddress, byte param0, float[] param1, float[] param2, byte param3, float[] param4);
	public static native void invokePPV(long __functionAddress, int param0, int param1, int param2, int[] param3, int[] param4);
	public static native void invokePV(long __functionAddress, int param0, int param1, double[] param2);
	public static native void invokePPPV(long __functionAddress, int param0, int param1, double[] param2, double[] param3, double[] param4);
	public static native void invokePV(long __functionAddress, int param0, int param1, float[] param2);
	public static native void invokePV(long __functionAddress, int param0, int param1, float[] param2, int param3, int param4);
	public static native void invokePPPV(long __functionAddress, int param0, int param1, float[] param2, float[] param3, float[] param4);
	public static native void invokePV(long __functionAddress, int param0, int param1, int[] param2);
	public static native void invokePV(long __functionAddress, int param0, int param1, int[] param2, int param3, int param4);
	public static native void invokePV(long __functionAddress, int param0, int param1, long[] param2);
	public static native void invokePPPV(long __functionAddress, int param0, int param1, long[] param2, long[] param3, long[] param4);
	public static native void invokePV(long __functionAddress, int param0, int param1, short[] param2, int param3, int param4);
	public static native void invokePPPPPV(long __functionAddress, int param0, long param1, int param2, float[] param3, float[] param4, long param5, int param6, long param7, int param8, boolean param9);
	public static native void invokePV(long __functionAddress, int param0, double[] param1);
	public static native void invokePV(long __functionAddress, int param0, float[] param1);
	public static native void invokePPPV(long __functionAddress, int param0, float[] param1, float[] param2, float[] param3);
	public static native void invokePV(long __functionAddress, int param0, int[] param1);
	public static native void invokePPPPPV(long __functionAddress, int param0, int[] param1, int param2, float[] param3, float[] param4, long param5, int param6, int[] param7, int param8, boolean param9);
	public static native void invokePPPPPV(long __functionAddress, int param0, short[] param1, int param2, float[] param3, float[] param4, long param5, int param6, short[] param7, int param8, boolean param9);
	public static native void invokePPV(long __functionAddress, long param0, int param1, int param2, int[] param3);
	public static native void invokePPPV(long __functionAddress, long param0, double[] param1, double[] param2);
	public static native void invokePPV(long __functionAddress, long param0, float[] param1, int param2);
	public static native void invokePPV(long __functionAddress, long param0, int[] param1, int param2);
	public static native void invokePPPV(long __functionAddress, long param0, int[] param1, int[] param2);
	public static native void invokePPPPPV(long __functionAddress, long param0, int[] param1, int[] param2, int[] param3, int[] param4);
	public static native void invokePPV(long __functionAddress, long param0, short[] param1, int param2);
	public static native void invokePV(long __functionAddress, short param0, double[] param1, short param2);
	public static native void invokePV(long __functionAddress, short param0, float[] param1, short param2);
	public static native void invokePV(long __functionAddress, short param0, int[] param1, short param2);
	public static native void invokePV(long __functionAddress, short param0, long[] param1, short param2);
	public static native void invokePV(long __functionAddress, short param0, short[] param1, short param2);
	public static native void invokePPPV(long __functionAddress, float[] param0, int param1, long param2, long param3, int param4);
	public static native void invokePPPV(long __functionAddress, float[] param0, boolean param1, int param2, long param3, long param4, int param5);
	public static native void invokePPPV(long __functionAddress, int[] param0, int[] param1, int[] param2);
	public static native boolean invokePPZ(long __functionAddress, long param0, int[] param1);
	public static native int callPI(long __functionAddress, int param0, int param1, int param2, int param3, float[] param4);
	public static native int callPI(long __functionAddress, int param0, int param1, int param2, int param3, int[] param4);
	public static native int callPI(long __functionAddress, int param0, int param1, int[] param2);
	public static native int callPPPPPI(long __functionAddress, int param0, int param1, int[] param2, int[] param3, int[] param4, int[] param5, long param6);
	public static native int callPPPPPPI(long __functionAddress, int param0, int param1, int[] param2, int[] param3, int[] param4, int[] param5, int[] param6, long param7);
	public static native int callPPI(long __functionAddress, int param0, long param1, int[] param2);
	public static native int callPI(long __functionAddress, int param0, int[] param1);
	public static native int callPPPI(long __functionAddress, long param0, int param1, int param2, int param3, int[] param4, float[] param5);
	public static native int callPPPI(long __functionAddress, long param0, int param1, int param2, int param3, int[] param4, int[] param5);
	public static native int callPPPPI(long __functionAddress, long param0, int param1, int param2, long param3, int[] param4, long param5);
	public static native int callPPPPI(long __functionAddress, long param0, int param1, int param2, long param3, long[] param4, long param5);
	public static native int callPPI(long __functionAddress, long param0, int param1, int param2, int[] param3);
	public static native int callPPPPI(long __functionAddress, long param0, int param1, long param2, int param3, int param4, long param5, int[] param6);
	public static native int callPPPPI(long __functionAddress, long param0, int param1, long param2, long param3, long[] param4);
	public static native int callPPPI(long __functionAddress, long param0, int param1, long param2, double[] param3);
	public static native int callPPPI(long __functionAddress, long param0, int param1, long param2, float[] param3);
	public static native int callPJPI(long __functionAddress, long param0, int param1, long param2, int[] param3);
	public static native int callPPPI(long __functionAddress, long param0, int param1, long param2, int[] param3);
	public static native int callPPPPI(long __functionAddress, long param0, int param1, long param2, int[] param3, long param4);
	public static native int callPPPI(long __functionAddress, long param0, int param1, long param2, long[] param3);
	public static native int callPPPPI(long __functionAddress, long param0, int param1, long param2, long[] param3, long param4);
	public static native int callPPPI(long __functionAddress, long param0, int param1, long param2, short[] param3);
	public static native int callPPI(long __functionAddress, long param0, int param1, int[] param2);
	public static native int callPPPI(long __functionAddress, long param0, int param1, int[] param2, int[] param3);
	public static native int callPPPI(long __functionAddress, long param0, int param1, int[] param2, long[] param3);
	public static native int callPPI(long __functionAddress, long param0, int param1, long[] param2);
	public static native int callPPJI(long __functionAddress, long param0, int param1, long[] param2, int param3, long param4);
	public static native int callPJPPI(long __functionAddress, long param0, long param1, int param2, int param3, long param4, int[] param5);
	public static native int callPJPPJI(long __functionAddress, long param0, long param1, int param2, int param3, long param4, int[] param5, long param6, int param7);
	public static native int callPJPPJI(long __functionAddress, long param0, long param1, int param2, int param3, long param4, long[] param5, long param6, int param7);
	public static native int callPPPPPPPPPPPPI(long __functionAddress, long param0, long param1, int param2, long param3, long param4, long param5, long param6, long param7, long param8, long param9, double[] param10, int param11, long param12, long param13);
	public static native int callPPPPPPPPPPPPI(long __functionAddress, long param0, long param1, int param2, long param3, long param4, long param5, long param6, long param7, long param8, long param9, float[] param10, int param11, long param12, long param13);
	public static native int callPPPPPPPPPPPPI(long __functionAddress, long param0, long param1, int param2, long param3, long param4, long param5, long param6, long param7, long param8, long param9, int[] param10, int param11, long param12, long param13);
	public static native int callPPPPPPPPPPPPI(long __functionAddress, long param0, long param1, int param2, long param3, long param4, long param5, long param6, long param7, long param8, long param9, short[] param10, int param11, long param12, long param13);
	public static native int callPPPPPPPPPI(long __functionAddress, long param0, long param1, int param2, long param3, long param4, long param5, long param6, double[] param7, int param8, long param9, long param10);
	public static native int callPPPPPPPPPI(long __functionAddress, long param0, long param1, int param2, long param3, long param4, long param5, long param6, float[] param7, int param8, long param9, long param10);
	public static native int callPPPPPPPPPI(long __functionAddress, long param0, long param1, int param2, long param3, long param4, long param5, long param6, int[] param7, int param8, long param9, long param10);
	public static native int callPPPPPPPPPI(long __functionAddress, long param0, long param1, int param2, long param3, long param4, long param5, long param6, short[] param7, int param8, long param9, long param10);
	public static native int callPPPPPPPI(long __functionAddress, long param0, long param1, int param2, long param3, long param4, double[] param5, int param6, long param7, long param8);
	public static native int callPPPPPPPI(long __functionAddress, long param0, long param1, int param2, long param3, long param4, float[] param5, int param6, long param7, long param8);
	public static native int callPPPPPPPI(long __functionAddress, long param0, long param1, int param2, long param3, long param4, int[] param5, int param6, long param7, long param8);
	public static native int callPJPPPI(long __functionAddress, long param0, long param1, int param2, long param3, long param4, long[] param5);
	public static native int callPPPPPPPI(long __functionAddress, long param0, long param1, int param2, long param3, long param4, short[] param5, int param6, long param7, long param8);
	public static native int callPJPPI(long __functionAddress, long param0, long param1, int param2, long param3, int[] param4);
	public static native int callPPPPI(long __functionAddress, long param0, long param1, int param2, long param3, int[] param4);
	public static native int callPPPPPI(long __functionAddress, long param0, long param1, int param2, long param3, int[] param4, long param5);
	public static native int callPPPPPI(long __functionAddress, long param0, long param1, int param2, long param3, long[] param4, long param5);
	public static native int callPPPI(long __functionAddress, long param0, long param1, int param2, int[] param3);
	public static native int callPPPI(long __functionAddress, long param0, long param1, int param2, long[] param3);
	public static native int callPJPI(long __functionAddress, long param0, long param1, int param2, long[] param3);
	public static native int callPPPPI(long __functionAddress, long param0, long param1, long param2, int param3, int[] param4);
	public static native int callPPPPPPPI(long __functionAddress, long param0, long param1, long param2, long param3, int param4, long param5, int[] param6, long param7);
	public static native int callPJJJJPI(long __functionAddress, long param0, long param1, long param2, long param3, long param4, int[] param5);
	public static native int callPJPPPI(long __functionAddress, long param0, long param1, long param2, long param3, long[] param4);
	public static native int callPPPPI(long __functionAddress, long param0, long param1, long param2, long[] param3);
	public static native int callPPPI(long __functionAddress, long param0, long param1, int[] param2);
	public static native int callPPPI(long __functionAddress, long param0, long param1, int[] param2, int param3);
	public static native int callPPPPI(long __functionAddress, long param0, long param1, int[] param2, long param3);
	public static native int callPJPPI(long __functionAddress, long param0, long param1, int[] param2, long param3);
	public static native int callPPPPI(long __functionAddress, long param0, long param1, int[] param2, int[] param3);
	public static native int callPJPPI(long __functionAddress, long param0, long param1, int[] param2, int[] param3);
	public static native int callPPPPPI(long __functionAddress, long param0, long param1, int[] param2, int[] param3, int[] param4);
	public static native int callPPPPPI(long __functionAddress, long param0, long param1, int[] param2, int[] param3, long[] param4);
	public static native int callPJPPI(long __functionAddress, long param0, long param1, int[] param2, long[] param3);
	public static native int callPPPI(long __functionAddress, long param0, long param1, long[] param2);
	public static native int callPPI(long __functionAddress, long param0, int[] param1);
	public static native int callPPPI(long __functionAddress, long param0, int[] param1, long param2);
	public static native int callPPPPI(long __functionAddress, long param0, int[] param1, long param2, int param3, int[] param4);
	public static native int callPPPPPI(long __functionAddress, long param0, int[] param1, float[] param2, int param3, int[] param4, int[] param5);
	public static native int callPPPI(long __functionAddress, long param0, int[] param1, int[] param2);
	public static native int callPPPPPI(long __functionAddress, long param0, int[] param1, int[] param2, int[] param3, long param4);
	public static native int callPPPPPPI(long __functionAddress, long param0, int[] param1, int[] param2, int[] param3, int[] param4, int[] param5);
	public static native int callPPI(long __functionAddress, long param0, long[] param1);
	public static native int callPPPPI(long __functionAddress, long param0, long[] param1, int param2, long param3, int[] param4);
	public static native int callPPPI(long __functionAddress, long param0, long[] param1, long[] param2);
	public static native int callPI(long __functionAddress, int[] param0);
	public static native int callPPI(long __functionAddress, int[] param0, long param1);
	public static native int callPPPI(long __functionAddress, int[] param0, long param1, int[] param2);
	public static native long callPPP(long __functionAddress, int param0, int param1, int param2, int[] param3, int[] param4);
	public static native long callPPP(long __functionAddress, int param0, long param1, int[] param2);
	public static native long callPPP(long __functionAddress, long param0, int param1, int param2, int param3, int[] param4);
	public static native long callPPPPPP(long __functionAddress, long param0, int param1, long param2, long param3, long param4, int[] param5);
	public static native long callPPPPPPP(long __functionAddress, long param0, int param1, long param2, long param3, long param4, int[] param5, int[] param6);
	public static native long callPPPPP(long __functionAddress, long param0, int param1, long param2, long param3, int[] param4);
	public static native long callPPP(long __functionAddress, long param0, int param1, int[] param2);
	public static native long callPPPP(long __functionAddress, long param0, int param1, int[] param2, long param3);
	public static native long callPJPP(long __functionAddress, long param0, long param1, int param2, int param3, int param4, int[] param5);
	public static native long callPPPP(long __functionAddress, long param0, long param1, int param2, int param3, int[] param4);
	public static native long callPJPPP(long __functionAddress, long param0, long param1, int param2, int param3, int[] param4, int[] param5);
	public static native long callPPJPPPPPP(long __functionAddress, long param0, long param1, int param2, long param3, long param4, long param5, int param6, long param7, long param8, int[] param9);
	public static native long callPPJPPPPPPPP(long __functionAddress, long param0, long param1, int param2, long param3, long param4, long param5, long param6, long param7, int param8, long param9, long param10, int[] param11);
	public static native long callPPPPP(long __functionAddress, long param0, long param1, int param2, long param3, int[] param4);
	public static native long callPJPPP(long __functionAddress, long param0, long param1, int param2, long param3, int[] param4);
	public static native long callPPPP(long __functionAddress, long param0, long param1, int param2, int[] param3);
	public static native long callPJPP(long __functionAddress, long param0, long param1, int param2, int[] param3);
	public static native long callPPPPP(long __functionAddress, long param0, long param1, long param2, int param3, int[] param4);
	public static native long callPJPPPPPPPPP(long __functionAddress, long param0, long param1, long param2, long param3, long param4, long param5, long param6, long param7, long param8, int[] param9);
	public static native long callPJPPPPPPPPP(long __functionAddress, long param0, long param1, long param2, long param3, long param4, long param5, long param6, long param7, float[] param8, int[] param9);
	public static native long callPJPPPPPPPPP(long __functionAddress, long param0, long param1, long param2, long param3, long param4, long param5, long param6, long param7, int[] param8, int[] param9);
	public static native long callPJPPPPPPPPP(long __functionAddress, long param0, long param1, long param2, long param3, long param4, long param5, long param6, long param7, short[] param8, int[] param9);
	public static native long callPJPPPPPPP(long __functionAddress, long param0, long param1, long param2, long param3, long param4, long param5, long param6, int[] param7);
	public static native long callPJPPPPPPP(long __functionAddress, long param0, long param1, long param2, long param3, long param4, long param5, float[] param6, int[] param7);
	public static native long callPJPPPPPPP(long __functionAddress, long param0, long param1, long param2, long param3, long param4, long param5, int[] param6, int[] param7);
	public static native long callPJPPPPPPP(long __functionAddress, long param0, long param1, long param2, long param3, long param4, long param5, short[] param6, int[] param7);
	public static native long callPPPJPPP(long __functionAddress, long param0, long param1, long param2, long param3, long param4, int[] param5);
	public static native long callPJPPPPP(long __functionAddress, long param0, long param1, long param2, long param3, long param4, int[] param5);
	public static native long callPJPPPPP(long __functionAddress, long param0, long param1, long param2, long param3, float[] param4, int[] param5);
	public static native long callPJPPPP(long __functionAddress, long param0, long param1, long param2, long param3, int[] param4);
	public static native long callPJPPPPP(long __functionAddress, long param0, long param1, long param2, long param3, int[] param4, int[] param5);
	public static native long callPJPPPPP(long __functionAddress, long param0, long param1, long param2, long param3, short[] param4, int[] param5);
	public static native long callPJPPPP(long __functionAddress, long param0, long param1, long param2, double[] param3, int[] param4);
	public static native long callPJPPPP(long __functionAddress, long param0, long param1, long param2, float[] param3, int[] param4);
	public static native long callPPPPP(long __functionAddress, long param0, long param1, long param2, int[] param3);
	public static native long callPPJPP(long __functionAddress, long param0, long param1, long param2, int[] param3);
	public static native long callPJPPPP(long __functionAddress, long param0, long param1, long param2, int[] param3, int[] param4);
	public static native long callPJPPPP(long __functionAddress, long param0, long param1, long param2, short[] param3, int[] param4);
	public static native long callPPPP(long __functionAddress, long param0, long param1, int[] param2);
	public static native long callPJPPP(long __functionAddress, long param0, long param1, int[] param2, int param3, int[] param4);
	public static native long callPPPPP(long __functionAddress, long param0, long param1, long[] param2, int[] param3);
	public static native long callPPP(long __functionAddress, long param0, int[] param1);
	public static native long callPPPP(long __functionAddress, long param0, int[] param1, int[] param2);
	public static native long callPP(long __functionAddress, int[] param0);
	public static native void callPV(long __functionAddress, int param0, double param1, double param2, int param3, int param4, double param5, double param6, int param7, int param8, double[] param9);
	public static native void callPV(long __functionAddress, int param0, double param1, double param2, int param3, int param4, double[] param5);
	public static native void callPV(long __functionAddress, int param0, float param1, float param2, int param3, int param4, float param5, float param6, int param7, int param8, float[] param9);
	public static native void callPV(long __functionAddress, int param0, float param1, float param2, int param3, int param4, float[] param5);
	public static native void callPV(long __functionAddress, int param0, int param1, int param2, int param3, int param4, int param5, int param6, int param7, int param8, int param9, int param10, double[] param11);
	public static native void callPV(long __functionAddress, int param0, int param1, int param2, int param3, int param4, int param5, int param6, int param7, int param8, int param9, int param10, float[] param11);
	public static native void callPV(long __functionAddress, int param0, int param1, int param2, int param3, int param4, int param5, int param6, int param7, int param8, int param9, int param10, int[] param11);
	public static native void callPV(long __functionAddress, int param0, int param1, int param2, int param3, int param4, int param5, int param6, int param7, int param8, int param9, int param10, short[] param11);
	public static native void callPV(long __functionAddress, int param0, int param1, int param2, int param3, int param4, int param5, int param6, int param7, int param8, int param9, double[] param10);
	public static native void callPV(long __functionAddress, int param0, int param1, int param2, int param3, int param4, int param5, int param6, int param7, int param8, int param9, float[] param10);
	public static native void callPV(long __functionAddress, int param0, int param1, int param2, int param3, int param4, int param5, int param6, int param7, int param8, int param9, int[] param10);
	public static native void callPV(long __functionAddress, int param0, int param1, int param2, int param3, int param4, int param5, int param6, int param7, int param8, int param9, short[] param10);
	public static native void callPV(long __functionAddress, int param0, int param1, int param2, int param3, int param4, int param5, int param6, int param7, int param8, double[] param9);
	public static native void callPV(long __functionAddress, int param0, int param1, int param2, int param3, int param4, int param5, int param6, int param7, int param8, float[] param9);
	public static native void callPV(long __functionAddress, int param0, int param1, int param2, int param3, int param4, int param5, int param6, int param7, int param8, int[] param9);
	public static native void callPV(long __functionAddress, int param0, int param1, int param2, int param3, int param4, int param5, int param6, int param7, int param8, short[] param9);
	public static native void callPV(long __functionAddress, int param0, int param1, int param2, int param3, int param4, int param5, int param6, int param7, double[] param8);
	public static native void callPV(long __functionAddress, int param0, int param1, int param2, int param3, int param4, int param5, int param6, int param7, float[] param8);
	public static native void callPV(long __functionAddress, int param0, int param1, int param2, int param3, int param4, int param5, int param6, int param7, int[] param8);
	public static native void callPV(long __functionAddress, int param0, int param1, int param2, int param3, int param4, int param5, int param6, int param7, short[] param8);
	public static native void callPV(long __functionAddress, int param0, int param1, int param2, int param3, int param4, int param5, int param6, double[] param7);
	public static native void callPV(long __functionAddress, int param0, int param1, int param2, int param3, int param4, int param5, int param6, float[] param7);
	public static native void callPV(long __functionAddress, int param0, int param1, int param2, int param3, int param4, int param5, int param6, int[] param7);
	public static native void callPV(long __functionAddress, int param0, int param1, int param2, int param3, int param4, int param5, int param6, short[] param7);
	public static native void callPV(long __functionAddress, int param0, int param1, int param2, int param3, int param4, int param5, double[] param6);
	public static native void callPV(long __functionAddress, int param0, int param1, int param2, int param3, int param4, int param5, float[] param6);
	public static native void callPV(long __functionAddress, int param0, int param1, int param2, int param3, int param4, int param5, int[] param6);
	public static native void callPV(long __functionAddress, int param0, int param1, int param2, int param3, int param4, int param5, short[] param6);
	public static native void callPV(long __functionAddress, int param0, int param1, int param2, int param3, int param4, double[] param5);
	public static native void callPV(long __functionAddress, int param0, int param1, int param2, int param3, int param4, float[] param5);
	public static native void callPV(long __functionAddress, int param0, int param1, int param2, int param3, int param4, int[] param5);
	public static native void callPV(long __functionAddress, int param0, int param1, int param2, int param3, int param4, short[] param5);
	public static native void callPPV(long __functionAddress, int param0, int param1, int param2, int param3, long param4, int param5, int param6, float[] param7);
	public static native void callPPV(long __functionAddress, int param0, int param1, int param2, int param3, long param4, int param5, int param6, short[] param7);
	public static native void callPV(long __functionAddress, int param0, int param1, int param2, int param3, double[] param4);
	public static native void callPV(long __functionAddress, int param0, int param1, int param2, int param3, float[] param4);
	public static native void callPV(long __functionAddress, int param0, int param1, int param2, int param3, int[] param4);
	public static native void callPPPV(long __functionAddress, int param0, int param1, int param2, int param3, int[] param4, int param5, int[] param6, float[] param7);
	public static native void callPPPV(long __functionAddress, int param0, int param1, int param2, int param3, int[] param4, int param5, int[] param6, int[] param7);
	public static native void callPPV(long __functionAddress, int param0, int param1, int param2, int param3, int[] param4, long param5);
	public static native void callPV(long __functionAddress, int param0, int param1, int param2, int param3, int[] param4, boolean param5);
	public static native void callPV(long __functionAddress, int param0, int param1, int param2, int param3, long[] param4);
	public static native void callPV(long __functionAddress, int param0, int param1, int param2, int param3, short[] param4);
	public static native void callPPV(long __functionAddress, int param0, int param1, int param2, long param3, int param4, float param5, float param6, int param7, float[] param8);
	public static native void callPPV(long __functionAddress, int param0, int param1, int param2, long param3, int param4, int param5, float[] param6);
	public static native void callPPPPPPPV(long __functionAddress, int param0, int param1, int param2, long param3, int param4, long param5, int[] param6, int[] param7, int[] param8, int[] param9, long[] param10);
	public static native void callPPV(long __functionAddress, int param0, int param1, int param2, long param3, int[] param4);
	public static native void callPV(long __functionAddress, int param0, int param1, int param2, boolean param3, int param4, float[] param5);
	public static native void callPV(long __functionAddress, int param0, int param1, int param2, boolean param3, int param4, int[] param5);
	public static native void callPV(long __functionAddress, int param0, int param1, int param2, boolean param3, int param4, short[] param5);
	public static native void callPV(long __functionAddress, int param0, int param1, int param2, boolean param3, double[] param4);
	public static native void callPV(long __functionAddress, int param0, int param1, int param2, boolean param3, float[] param4);
	public static native void callPV(long __functionAddress, int param0, int param1, int param2, double[] param3);
	public static native void callPV(long __functionAddress, int param0, int param1, int param2, float[] param3);
	public static native void callPV(long __functionAddress, int param0, int param1, int param2, int[] param3);
	public static native void callPPV(long __functionAddress, int param0, int param1, int param2, int[] param3, long param4);
	public static native void callPPPV(long __functionAddress, int param0, int param1, int param2, int[] param3, long param4, long param5);
	public static native void callPPPV(long __functionAddress, int param0, int param1, int param2, int[] param3, long param4, int[] param5);
	public static native void callPV(long __functionAddress, int param0, int param1, int param2, int[] param3, boolean param4);
	public static native void callPPV(long __functionAddress, int param0, int param1, int param2, int[] param3, int[] param4);
	public static native void callPPPPV(long __functionAddress, int param0, int param1, int param2, int[] param3, int[] param4, int[] param5, long param6);
	public static native void callPV(long __functionAddress, int param0, int param1, int param2, long[] param3);
	public static native void callPV(long __functionAddress, int param0, int param1, int param2, short[] param3);
	public static native void callPPV(long __functionAddress, int param0, int param1, long param2, int param3, int param4, int param5, int param6, int param7, float[] param8);
	public static native void callPPV(long __functionAddress, int param0, int param1, long param2, int param3, int param4, int param5, int param6, float[] param7);
	public static native void callPPV(long __functionAddress, int param0, int param1, long param2, int param3, int param4, int param5, float[] param6);
	public static native void callPPV(long __functionAddress, int param0, int param1, long param2, int param3, int param4, float[] param5);
	public static native void callPPV(long __functionAddress, int param0, int param1, long param2, int param3, int param4, short[] param5);
	public static native void callPPPV(long __functionAddress, int param0, int param1, long param2, long param3, int param4, int param5, float[] param6);
	public static native void callPPPV(long __functionAddress, int param0, int param1, long param2, long param3, int param4, int param5, int[] param6);
	public static native void callPPPV(long __functionAddress, int param0, int param1, long param2, long param3, int param4, int param5, short[] param6);
	public static native void callPPPV(long __functionAddress, int param0, int param1, long param2, long param3, double[] param4);
	public static native void callPPPV(long __functionAddress, int param0, int param1, long param2, long param3, float[] param4);
	public static native void callPPPV(long __functionAddress, int param0, int param1, long param2, long param3, int[] param4);
	public static native void callPPPPV(long __functionAddress, int param0, int param1, long param2, long param3, int[] param4, int[] param5, int param6);
	public static native void callPPPV(long __functionAddress, int param0, int param1, long param2, long param3, short[] param4);
	public static native void callPPV(long __functionAddress, int param0, int param1, long param2, int[] param3);
	public static native void callPPV(long __functionAddress, int param0, int param1, long param2, int[] param3, int param4);
	public static native void callPPPPPV(long __functionAddress, int param0, int param1, long param2, int[] param3, int[] param4, int[] param5, int[] param6);
	public static native void callPV(long __functionAddress, int param0, int param1, boolean param2, double[] param3);
	public static native void callPV(long __functionAddress, int param0, int param1, boolean param2, float[] param3);
	public static native void callPV(long __functionAddress, int param0, int param1, boolean param2, int[] param3);
	public static native void callPV(long __functionAddress, int param0, int param1, double[] param2);
	public static native void callPV(long __functionAddress, int param0, int param1, float[] param2);
	public static native void callPV(long __functionAddress, int param0, int param1, int[] param2);
	public static native void callPV(long __functionAddress, int param0, int param1, int[] param2, int param3);
	public static native void callPV(long __functionAddress, int param0, int param1, int[] param2, int param3, int param4);
	public static native void callPV(long __functionAddress, int param0, int param1, int[] param2, int param3, int param4, int param5, int param6);
	public static native void callPPV(long __functionAddress, int param0, int param1, int[] param2, int param3, int[] param4);
	public static native void callPPV(long __functionAddress, int param0, int param1, int[] param2, long param3);
	public static native void callPPV(long __functionAddress, int param0, int param1, int[] param2, long param3, int param4, int param5);
	public static native void callPPPV(long __functionAddress, int param0, int param1, int[] param2, long param3, int[] param4);
	public static native void callPPV(long __functionAddress, int param0, int param1, int[] param2, float[] param3);
	public static native void callPPV(long __functionAddress, int param0, int param1, int[] param2, int[] param3);
	public static native void callPPPV(long __functionAddress, int param0, int param1, int[] param2, int[] param3, long param4);
	public static native void callPV(long __functionAddress, int param0, int param1, long[] param2);
	public static native void callPV(long __functionAddress, int param0, int param1, short[] param2);
	public static native void callPPV(long __functionAddress, int param0, long param1, int param2, int[] param3);
	public static native void callPPPV(long __functionAddress, int param0, long param1, int param2, int[] param3, long param4);
	public static native void callPPPV(long __functionAddress, int param0, long param1, int param2, int[] param3, int[] param4);
	public static native void callPPPV(long __functionAddress, int param0, long param1, long param2, double[] param3);
	public static native void callPPPV(long __functionAddress, int param0, long param1, long param2, float[] param3);
	public static native void callPPPV(long __functionAddress, int param0, long param1, long param2, int[] param3);
	public static native void callPPPV(long __functionAddress, int param0, long param1, long param2, short[] param3);
	public static native void callPPV(long __functionAddress, int param0, long param1, double[] param2, int param3);
	public static native void callPPV(long __functionAddress, int param0, long param1, float[] param2, int param3);
	public static native void callPPV(long __functionAddress, int param0, long param1, int[] param2, int param3);
	public static native void callPPPPV(long __functionAddress, int param0, long param1, int[] param2, int[] param3, int[] param4, int param5);
	public static native void callPPV(long __functionAddress, int param0, long param1, short[] param2, int param3);
	public static native void callPV(long __functionAddress, int param0, boolean param1, int param2, int param3, int[] param4);
	public static native void callPV(long __functionAddress, int param0, double[] param1);
	public static native void callPV(long __functionAddress, int param0, float[] param1);
	public static native void callPV(long __functionAddress, int param0, int[] param1);
	public static native void callPV(long __functionAddress, int param0, int[] param1, int param2);
	public static native void callPV(long __functionAddress, int param0, int[] param1, int param2, int param3);
	public static native void callPPV(long __functionAddress, int param0, int[] param1, int param2, long param3, int param4);
	public static native void callPPPV(long __functionAddress, int param0, int[] param1, int param2, long param3, int param4, int[] param5);
	public static native void callPPV(long __functionAddress, int param0, int[] param1, int param2, int[] param3, int param4);
	public static native void callPPV(long __functionAddress, int param0, int[] param1, long param2, int param3, int param4);
	public static native void callPPV(long __functionAddress, int param0, int[] param1, float[] param2);
	public static native void callPPV(long __functionAddress, int param0, int[] param1, int[] param2);
	public static native void callPPV(long __functionAddress, int param0, int[] param1, int[] param2, int param3);
	public static native void callPPPV(long __functionAddress, int param0, int[] param1, int[] param2, int param3, int[] param4);
	public static native void callPV(long __functionAddress, int param0, long[] param1);
	public static native void callPPV(long __functionAddress, int param0, long[] param1, int[] param2, int param3);
	public static native void callPV(long __functionAddress, int param0, short[] param1);
	public static native void callPPPV(long __functionAddress, long param0, int param1, int param2, int param3, int param4, int param5, int[] param6, long param7);
	public static native void callPPPV(long __functionAddress, long param0, int param1, int param2, int[] param3, int[] param4);
	public static native void callPPPV(long __functionAddress, long param0, int param1, int param2, long[] param3, long[] param4);
	public static native void callPJPPV(long __functionAddress, long param0, int param1, long param2, int param3, int param4, long[] param5, int param6, int[] param7);
	public static native void callPPPV(long __functionAddress, long param0, int param1, int[] param2, long param3);
	public static native void callPPPPPV(long __functionAddress, long param0, int param1, long[] param2, int param3, int param4, int param5, long param6, int param7, long param8, int param9, long param10);
	public static native void callPJPV(long __functionAddress, long param0, long param1, int param2, int param3, int param4, double[] param5);
	public static native void callPJPV(long __functionAddress, long param0, long param1, int param2, int param3, int param4, float[] param5);
	public static native void callPJPV(long __functionAddress, long param0, long param1, int param2, int param3, int param4, int[] param5);
	public static native void callPJPV(long __functionAddress, long param0, long param1, int param2, int param3, int param4, long[] param5);
	public static native void callPJPV(long __functionAddress, long param0, long param1, int param2, int param3, int param4, short[] param5);
	public static native void callPPPV(long __functionAddress, long param0, long param1, int param2, int[] param3);
	public static native void callPJJJPV(long __functionAddress, long param0, long param1, long param2, long param3, double[] param4);
	public static native void callPJJJPV(long __functionAddress, long param0, long param1, long param2, long param3, float[] param4);
	public static native void callPJJJPV(long __functionAddress, long param0, long param1, long param2, long param3, int[] param4);
	public static native void callPJJJPV(long __functionAddress, long param0, long param1, long param2, long param3, long[] param4);
	public static native void callPJJJPV(long __functionAddress, long param0, long param1, long param2, long param3, short[] param4);
	public static native void callPJPPV(long __functionAddress, long param0, long param1, int[] param2, long param3);
	public static native void callPJPV(long __functionAddress, long param0, long param1, long[] param2);
	public static native void callPPV(long __functionAddress, long param0, float[] param1);
	public static native void callPPV(long __functionAddress, long param0, int[] param1);
	public static native void callPPPV(long __functionAddress, long param0, int[] param1, long param2);
	public static native void callPV(long __functionAddress, double[] param0);
	public static native void callPPV(long __functionAddress, double[] param0, double[] param1);
	public static native void callPV(long __functionAddress, float[] param0);
	public static native void callPPV(long __functionAddress, float[] param0, float[] param1);
	public static native void callPV(long __functionAddress, int[] param0);
	public static native void callPPV(long __functionAddress, int[] param0, int param1, int[] param2);
	public static native void callPPV(long __functionAddress, int[] param0, int[] param1);
	public static native void callPPPPV(long __functionAddress, long[] param0, int[] param1, int[] param2, int[] param3, int param4);
	public static native void callPV(long __functionAddress, short[] param0);
	public static native void callPPV(long __functionAddress, short[] param0, short[] param1);
	public static native boolean callPPPPZ(long __functionAddress, int param0, int param1, int param2, float param3, float[] param4, float[] param5, float[] param6, float[] param7);
	public static native boolean callPPZ(long __functionAddress, int param0, int[] param1, long param2);

}
